package de.java2html.eclipse.preferences;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import de.java2html.eclipse.Java2HtmlPlugin;
import de.java2html.javasource.JavaSourceType;
import de.java2html.options.JavaSourceConversionOptions;
import de.java2html.options.JavaSourceStyleTable;

import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

public class Java2HtmlStylePreferencePage extends PreferencePage
    implements
    IWorkbenchPreferencePage,
    SelectionListener,
    SourceTypeSelectionListener,
    PropertyChangeListener {
  private Combo cbDefaultStyle;
  private List codeTypeList;
  private JavaSourceStyleEntryControl styleControl;
  private StylePreview preview;
  private JavaSourceConversionOptions options;
  private JavaSourceConversionOptions optionsBackup;

  public Java2HtmlStylePreferencePage() {
    super();
  }

  public void init(IWorkbench workbench) {
    //nothing to do
  }

  public Control createContents(Composite parent) {
    Java2HtmlPlugin plugin = getPlugin();
    if (plugin == null) {
      options = JavaSourceConversionOptions.getDefault().getClone();
      optionsBackup = JavaSourceConversionOptions.getDefault();
    }
    else {
      options = plugin.getConversionOptions().getClone();
      optionsBackup = plugin.getConversionOptions();
    }

    Composite top = new Composite(parent, SWT.LEFT);
    top.setLayoutData(new GridData(GridData.FILL_BOTH));
    top.setLayout(new GridLayout());

    //Liste aller verfgbaren Datentypen   
    Label listLabel = new Label(top, SWT.NONE);
    listLabel.setText("Styles:");

    Composite comp = new Composite(top, SWT.NONE);
    GridLayout gridLayout = new GridLayout(2, false);
    gridLayout.marginHeight = 0;
    gridLayout.marginWidth = 0;
    comp.setLayout(gridLayout);
    comp.setLayoutData(new GridData(GridData.FILL_BOTH));

    codeTypeList = new List(comp, SWT.BORDER | SWT.V_SCROLL);
    codeTypeList.setLayoutData(new GridData(GridData.FILL_BOTH));
    JavaSourceType[] allTypes = JavaSourceType.getAll();
    for (int i = 0; i < allTypes.length; ++i) {
      if (allTypes[i].isDisplayRelevant() || allTypes[i] == JavaSourceType.BACKGROUND) {
        codeTypeList.add(allTypes[i].getName());
      }
    }
    codeTypeList.select(0);
    codeTypeList.addSelectionListener(this);

    styleControl = new JavaSourceStyleEntryControl();
    styleControl.addPropertyChangeListener(this);
    Control control = styleControl.createControl(comp);
    control.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_BEGINNING));

    //Preview des Syntaxhighlighting   
    Label previewLabel = new Label(top, SWT.NONE);
    previewLabel.setText("Preview:");
    preview = new StylePreview();
    preview.createControl(top);
    preview.addSourceTypeSelectionListener(this);
    preview.setOptions(options);

    updateControls();
    return top;
  }

  protected void contributeButtons(Composite parent) {
    Label lDefaultStyle = new Label(parent, SWT.NONE);
    lDefaultStyle.setText("Default style:");
    cbDefaultStyle = new Combo(parent, SWT.DROP_DOWN | SWT.READ_ONLY);
    JavaSourceStyleTable[] tables = JavaSourceStyleTable.getPredefinedTables();
    String defaultName = getDefaultTableStyleName();
    for (int i = 0; i < tables.length; ++i) {
      cbDefaultStyle.add(tables[i].getName());
      if (tables[i].getName().equalsIgnoreCase(defaultName)) {
        cbDefaultStyle.select(i);
      }
    }
    ((GridLayout) parent.getLayout()).numColumns += 2;
  }

  private String getDefaultTableStyleName() {
    Java2HtmlPlugin plugin = getPlugin();
    return plugin == null ? "Eclipse" : plugin.getDefaultStyleTableName();
  }

  private void updateControls() {
    String propertyName = codeTypeList.getSelection()[0];
    styleControl.setJavaSourceStyleEntry(options.getStyleTable().get(propertyName));
  }

  private Java2HtmlPlugin getPlugin() {
    return Java2HtmlPlugin.getDefault();
  }

  public void widgetSelected(SelectionEvent event) {
    if (event.getSource() == codeTypeList) {
      updateControls();
    }
  }

  public void widgetDefaultSelected(SelectionEvent event) {
    //nothing to do
  }

  public boolean performCancel() {
    getPlugin().setConversionOptions(optionsBackup);
    return true;
  }

  public boolean performOk() {
    getPlugin().setConversionOptions(options);
    getPlugin().setDefaultStyleTableName(cbDefaultStyle.getItem(cbDefaultStyle.getSelectionIndex()));
    return true;
  }

  protected void performDefaults() {
    String tableName = cbDefaultStyle.getItem(cbDefaultStyle.getSelectionIndex());
    options.setStyleTable(JavaSourceStyleTable.getPredefinedTable(tableName));
    getPlugin().setConversionOptions(options);
    updateControls();
    preview.setOptions(options);
  }

  public void selectionChanged(SourceTypeSelectionEvent sourceTypeSelectionEvent) {
    for (int i = 0; i < codeTypeList.getItemCount(); ++i) {
      if (codeTypeList.getItem(i).equals(sourceTypeSelectionEvent.getJavaSourceType().getName())) {
        codeTypeList.select(i);
        codeTypeList.showSelection();
        updateControls();
        break;
      }
    }
  }

  public void propertyChange(PropertyChangeEvent evt) {
    String propertyName = codeTypeList.getSelection()[0];
    options.getStyleTable().put(propertyName, styleControl.getJavaSourceStyleEntry());
    preview.setOptions(options);
  }

  public void dispose() {
    super.dispose();
    preview.dispose();
  }
}