package de.java2html.eclipse.util;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import org.eclipse.core.internal.resources.Folder;
import org.eclipse.core.internal.resources.Project;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jdt.internal.core.Openable;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;

/**
 * @author Markus Gebhard
 */
public class EclipseUtilities {
  public static void writeTextToPromptedFile(String text) {
    Shell shell = new Shell();
    FileDialog dialog = new FileDialog (shell, SWT.SAVE);
    dialog.setFilterNames (new String [] {"HTML Files", "*.html"});
    dialog.setFilterExtensions (new String [] {"*.html", "*.*"}); //Windows wild cards
    //dialog.setFilterPath ("c:\\"); //Windows path
    dialog.setFileName ("java2html.html");
    String fileName = dialog.open();
    if (fileName!=null){
      BufferedWriter writer = null;
      try {
        writer = new BufferedWriter(new FileWriter(fileName));
        writer.write(text);
      } catch (IOException e) {
        MessageDialog.openError(shell, "Output error", "Error writing file: "+fileName);
      } finally {
        if (writer!=null){
          try{ 
            writer.close();
          }catch(Exception e){
          }
        }
      }
    }
  }

  public static File promptForSaveFile(String defaultName, String[] filterNames, String[] filterExtensions) {
    Shell shell = new Shell();
    FileDialog dialog = new FileDialog (shell, SWT.SAVE);
    dialog.setFilterNames (filterNames);
    dialog.setFilterExtensions (filterExtensions);
    dialog.setFileName (defaultName);
    String fileName = dialog.open();
    if (fileName!=null){
      return new File(fileName);
    }
    return null;
  }

  public static String getName(IAdaptable adaptable) {
    if (adaptable instanceof Folder){
      return ((Folder)adaptable).getName();
    }else
    if (adaptable instanceof Project){
      return ((Project)adaptable).getName();
    }else
    if (adaptable instanceof org.eclipse.core.internal.resources.File){
      String name = ((org.eclipse.core.internal.resources.File)adaptable).getName();
      if (name.endsWith(".java")){
        name = name.substring(0, name.length()-5);
      }
      return name;
    }else
//    if (adaptable instanceof JavaProject){
//      return ((JavaProject)adaptable).getElementName();
//    }else
    if (adaptable instanceof Openable){
      return ((Openable)adaptable).getElementName();
//      return c.getElementName();    
//      return ((CompilationUnit)adaptable).getElementName();
    }
//    else
//    if (adaptable instanceof PackageFragment){
//      return ((PackageFragment)adaptable).getElementName();
//    }
        
    return null;
  }
  
  public static void out(Shell shell, String text){
    MessageDialog.openInformation(
       shell,
       "Java2Html Message",
       text);
  }

  public static Color awtColor2swtColor(java.awt.Color color) {
    return new Color(null, color.getRed(), color.getGreen(), color.getBlue());
  }
}